package com.gct.driver.core;

import com.gct.driver.lotpi.DataBit;
import com.gct.driver.lotpi.DataType;
import com.gct.driver.lotpi.exception.DataBitInvalid;
import com.gct.driver.lotpi.exception.ReadException;
import com.sun.xml.internal.ws.util.StringUtils;

import java.util.Objects;

/**
 * dec:
 * <p>
 * BasicDataBit
 * </p>
 *
 * @author lloyd
 * @version 1.0.1
 * @since 2022/8/18 下午1:52
 */
public final class BasicDataBit extends DataBit {

    private DataBit parent;
    private Object value;
    private int dataNumbers = -1;
    private boolean isArray = false;


    public BasicDataBit(DataType type, Object address) {
        super(type, address);
    }

    public BasicDataBit(DataType type, Object address, int limitDataCount) {
        super(type, address, limitDataCount);
    }

    public BasicDataBit(DataType typeDefine, Object low, Object high) {
        super(typeDefine, low, high);
    }

    public BasicDataBit(DataType typeDefine, Object low, Object high, int limitDataCount) {
        super(typeDefine, low, high, limitDataCount);
    }

    public BasicDataBit(DataBit parent, ValueType valueType) throws ReadException {
        super(parent.getTypeDefine(), parent.getBitAddress(), parent.getHighBitAddress(), parent.getLimitDataCount());
        this.parent = parent;
        setSelfProp(valueType);
    }

    /**
     * 设定私有属性
     *
     * @param valueType
     */
    private void setSelfProp(ValueType valueType) {
        if (Objects.isNull(this.bitAddress)) throw new DataBitInvalid();

        if (!Objects.isNull(this.highBitAddress) && this.getTypeDefine() != DataType.BCD) {
            this.isArray = true;
            /**
             * 根据传入的类型和高低位地址，以及关联的相关通信类型计算dataNumbers
             */
            this.dataNumbers = Utilities.getDataNumber(this.bitAddress, this.highBitAddress, this.typeDefine, valueType);
            if (this.limitDataCount > 0) this.dataNumbers = Math.min(this.dataNumbers, this.limitDataCount);
        }
    }

    public DataBit getParent() {
        return parent;
    }

    public Object getValue() {
        return value;
    }

    public int getDataNumbers() {
        return dataNumbers;
    }

    public boolean isArray() {
        return isArray;
    }

    public void setValue(Object value) {
        this.value = value;
    }

    public static BasicDataBit[] transFromArray(DataBit[] dataBit, ValueType valueType) {
        BasicDataBit[] bdbs = new BasicDataBit[dataBit.length];
        for (int i = 0; i < dataBit.length; i++) {
            bdbs[i] = new BasicDataBit(dataBit[i], valueType);
        }
        return bdbs;
    }
}
