package com.gct.driver.core;


import com.gct.driver.core.operate.OperateResult;
import com.gct.driver.core.operate.OperateResultExOne;

/**
 * 所有设备类都需要实现此接口
 *
 */
public interface Device {

    /**
     * 与设备建立连接
     * @return true为成功
     */
    boolean Connect();

    /**
     * 连接状态
     * @return true为连接，false为断开
     */
    boolean ConnectStatus();

    /**
     * 断开连接
     * @return true为成功
     */
    boolean Close();

    /**
     * 读字节
     * @param address 开始地址地址
     * @param length  读取长度
     * @return 结果对象
     */
    OperateResultExOne<byte[]> Read(String address, short length);
    OperateResultExOne<byte[]> Read(String address);

    /**
     * 读位
     * @param address 开始地址地址
     * @param length  读取长度
     * @return 结果对象
     */
    OperateResultExOne<boolean[]> ReadBool(String address, short length);
    OperateResultExOne<Boolean> ReadBool(String address);


    /**
     * 写字节
     * @param address 开始地址地址
     * @param value  数值
     * @return 结果对象
     */
    OperateResult Write(String address, byte[] value);


//    /**
//     * 读位
//     * @param address 开始地址地址
//     * @param value  数值
//     * @return 结果对象
//     */
//    OperateResult WriteBool(String address, boolean[] value);
    /**
     * 读位
     * @param address 开始地址地址
     * @param value  数值
     * @return 结果对象
     */
    OperateResult WriteBool(String address, boolean  value);


}
